#include <stdint.h>
#include <stdio.h>
#include <QImage>
#include <QImageIOHandler>

#include <flif.h>


class FlifHandler: public QImageIOHandler {
public:
  FlifHandler() {}

  bool canRead() const { return canRead(device()); }
  static bool canRead( QIODevice *device );

  bool read( QImage *image );
  QByteArray name() const { return "flif"; }
};

bool FlifHandler::canRead( QIODevice *device ) {
  return device->peek(4) == "FLIF";
}

static inline uint32_t bgr_to_rgb( uint32_t rgb ) {
  return ((rgb << 16) & 0xff0000) | ((rgb >> 16) & 0xff) | (rgb & 0xff00ff00);
}

bool FlifHandler::read( QImage *image ) {
  FLIF_DECODER *dec = NULL;
  FLIF_IMAGE *img;
  int w, h;
  bool ret = true;

  QByteArray buf = device()->readAll();

  dec = flif_create_decoder();
  flif_decoder_set_quality( dec, 100 );
  flif_decoder_set_scale( dec, 1 );

  if (flif_decoder_decode_memory(dec, (uint8_t*)buf.data(), buf.size()) < 0) {
  	ret = false;
  	goto EXIT;
  }
  
  img = flif_decoder_get_image( dec, 0 );
  w = flif_image_get_width( img );
  h = flif_image_get_height( img );

  *image = *new QImage( w, h, QImage::Format_ARGB32 );

  for (int y = 0; y < h; ++y) {
  	uint32_t *scanLine = (uint32_t*)image->scanLine(y);
	flif_image_read_row_RGBA8( img, y, scanLine, w * sizeof(uint32_t) );
	for ( int x = 0; x < w; x++ )
		scanLine[x] = bgr_to_rgb( scanLine[x] );
  }

EXIT:
  if (dec) flif_destroy_decoder( dec );

  return ret;
}

class FlifPlugin: public QImageIOPlugin {
public:
  QStringList keys() const { return QStringList() << "flif"; }

  Capabilities capabilities(QIODevice *device, const QByteArray &format) const;
  QImageIOHandler *create(QIODevice *device, const QByteArray &format = QByteArray()) const;
};

QImageIOPlugin::Capabilities FlifPlugin::capabilities( QIODevice *device, const QByteArray &format ) const {

  if ( format == "flif" ) return Capabilities(CanRead);
  if ( !format.isEmpty() || !device->isOpen() ) return 0;

  if ( device->isReadable() && FlifHandler::canRead(device) )
  	return Capabilities(CanRead);

  return 0;
}

QImageIOHandler *FlifPlugin::create( QIODevice *device, const QByteArray &format ) const {
  FlifHandler *handler = new FlifHandler();
  handler->setDevice( device );
  handler->setFormat( format );
  return handler;
}

Q_EXPORT_STATIC_PLUGIN(FlifPlugin)
Q_EXPORT_PLUGIN2(flif, FlifPlugin)
